# Copyright (c) Ecometrica. All rights reserved.
# Distributed under the BSD license. See LICENSE for details.
# run with
# DJANGO_SETTINGS_MODULE=wapiti.settings python tests/__init__.py

from test_parsers import *

if __name__ == '__main__':
    unittest.main()

